package PlayBot::utils::db::get;

use strict;
use warnings;
use utf8;

use feature 'state';
use Moose;

use PlayBot::utils::db;


has '_dbh' => (
    is          => 'rw',
    isa         => 'HashRef',
    init_arg    => undef,
    default     => sub { state %h; return \%h; }
);

has '_sth' => (
    is          => 'rw',
    isa         => 'HashRef',
    init_arg    => undef,
    default     => sub { state %h; return \%h; }
);

has '_queries' => (
    is          => 'rw',
    isa         => 'HashRef',
    init_arg    => undef,
    default     => sub { state %h; return \%h; }
);

sub get {
    my ($self, $query) = @_;
    my $chan = $query->chan;
    my $result;

    if (defined $self->_queries->{$chan} and $self->_queries->{$chan} ~~ $query) {
        if ($self->_queries->{$chan}->date < time - 3600*6) {
            $self->_clean($chan);
            $self->_init($query);
            $self->_queries->{$chan} = $query;
        }

        $query = $self->_queries->{$chan};
    }
    else {
        $self->_init($query);
        $self->_queries->{$chan} = $query;
    }

    $result = $self->_get_next($query);

    # if id was precised, we (and so the user) know there is no more results
    $self->_clean($chan) if ($query->id >= 0);

    return undef unless $result;

    my %content;
    $content{'id'} = $result->[0];
    $content{'author'} = $result->[1];
    $content{'title'} = $result->[2];
    $content{'url'} = $result->[3];
    $content{'duration'} = $result->[4];
    $content{'external_id'} = $result->[5];
    $content{'site'} = $result->[6];
    return \%content;
}

sub get_rows {
    my ($self, $query) = @_;

    my $dbh = PlayBot::utils::db::get_new_session();
    my ($request, @args) = $self->_prepare_request($query);

    my $sth = $dbh->prepare('select count(*) from ('.$request.') as TGTGTG');
    $sth->execute(@args);

    my $rows = $sth->fetch->[0];
    $sth->finish;
    $dbh->commit;
    $dbh->disconnect;

    return $rows;
}

sub _get_next {
    my ($self, $query) = @_;
    my $chan = $query->chan;

    my $result = $self->_sth->{$chan}->fetch();
    return $result if ($result);

    # we come here only if there is no more data to fetch
    $self->_clean($chan);
    return undef;
}

sub _clean
{
    my ($self, $chan) = @_;

    # we take care of the db
    delete $self->_sth->{$chan};
    $self->_get_dbh($chan)->commit();

    # we remove the query
    delete $self->_queries->{$chan};
}

sub _init {
    my ($self, $query) = @_;
    my $chan = $query->chan;

    if (defined $self->_sth->{$chan}) {
        $self->_sth->{$chan}->finish();
    }

    my ($request, @args) = $self->_prepare_request($query);
    my $sth = $self->_get_dbh($chan)->prepare($request);
    $sth->execute(@args);

    $self->_sth->{$query->chan} = $sth;
}

sub _prepare_request {
    my ($self, $query) = @_;
    
    my @words_param;
    my $req;
    my @args;

    foreach (@{$query->words}) {
        unshift @words_param, '%'.$_.'%';
    }

    my $words_sql;
    foreach (@{$query->words}) {
        $words_sql .= ' and ' if ($words_sql);
        $words_sql .= "concat(p.sender, ' ', p.title) like ?";
    }

    if ($query->id >= 0) {
        $req = 'select p.id, p.sender, p.title, p.url, p.duration, p.external_id, p.type';
        $req .= ' from playbot p where id = ?';

        @args = ($query->id);
    }
    elsif (@{$query->tags}) {
        my @where;

        foreach my $tag (@{$query->tags}) {
            unshift @where, 'p.id in (select pt.id from playbot_tags pt where pt.tag = ?)';
        }

        my $where = join ' and ' => @where;

        if ($query->is_global) {
            $req = 'select p.id, p.sender, p.title, p.url, p.duration, p.external_id, p.type';
            $req .= ' from playbot p where '.$where;
            $req .= ' and '.$words_sql if ($words_sql);
            $req .= ' and p.playlist is false';
            $req .= ' group by p.id order by rand()';

            @args = (@{$query->tags}, @words_param);
        }
        else {
            $req = 'select p.id, p.sender, p.title, p.url, p.duration, p.external_id, p.type';
            $req .= ' from playbot p join playbot_chan pc on p.id = pc.content';
            $req .= ' where '.$where;
            $req .= ' and '.$words_sql if ($words_sql);
            $req .= ' and p.playlist is false';
            $req .= ' and pc.chan = ? group by p.id order by rand()';

            @args = (@{$query->tags}, @words_param, $query->chan);
        }
    }
    else {
        if ($query->is_global) {
            $req = 'select p.id, p.sender, p.title, p.url, p.duration, p.external_id, p.type';
            $req .= ' from playbot p';
            $req .= ' where '.$words_sql.' and' if ($words_sql);
            $req .= ' p.playlist is false';
            $req .= ' group by p.id order by rand()';

            @args = (@words_param);
        }
        else {
            $req = 'select p.id, p.sender, p.title, p.url, p.duration, p.external_id, p.type';
            $req .= ' from playbot p join playbot_chan pc on p.id = pc.content';
            $req .= ' where pc.chan = ?';
            $req .= ' and '.$words_sql if ($words_sql);
            $req .= ' and p.playlist is false';
            $req .= ' group by p.id order by rand()';

            @args = ($query->chan, @words_param);
        }
    }

    return ($req, @args);
}

sub _get_dbh {
    my ($self, $chan) = @_;

    if (not defined $self->_dbh->{$chan} or
        not $self->_dbh->{$chan}->ping
    ) {
        $self->_dbh->{$chan} = PlayBot::utils::db::get_new_session();
    }

    return $self->_dbh->{$chan};
}

1;
